require 'spec_helper'
require 'ey/logger/faraday'

describe 'Ey::Logger::Farday' do
  let!(:output) { StringIO.new }
  let!(:device) {
    logger = Logger.new(output)
    logger.formatter = proc { |_, _, _, msg| "#{msg}\n" }
    logger
  }
  let!(:connection) {
    Faraday.new { |builder|
      builder.use Ey::Logger::Faraday, device: device, format: format
      builder.adapter(:rack, lambda { |env| [200, {"Content-Type" => "application/json"}, ["{}"]] })
    }
  }

  context "with machine formatting" do
    let(:format) { :machine }

    it "should record request data" do
      connection.post do |req|
        req.url 'http://nigiri'
        req.headers['Content-Type'] = 'application/json'
        req.body = '{ "name": "Unagi" }'
      end
      request_line, response_line = output.tap(&:rewind).read.split("\n")

      request_output  = Ey::Logger::MachineParser.parse(request_line)

      expect(request_output.tag).to eq("ey.faraday.request")
      expect(request_output.data).to eq({
        "method"  => "post",
        "url"     => "http://nigiri",
        "headers" => {"User-Agent" => "Faraday v0.9.1", "Content-Type" => "application/json"},
        "body"    => "{ \"name\": \"Unagi\" }",
      })

      response_output = Ey::Logger::MachineParser.parse(response_line)

      expect(response_output.tag).to eq("ey.faraday.response")
      expect(response_output.data).to eq({
        "status"  => 200,
        "method"  => "post",
        "url"     => "http://nigiri",
        "body"    => "{}",
        "headers" => {"Content-Type"  => "application/json", "Content-Length" => "2"},
      })
    end
  end

  context "with human formatting" do
    let(:format) { :human }

    it "should record request data" do
      connection.post do |req|
        req.url 'http://nigiri'
        req.headers['Content-Type'] = 'application/json'
        req.body = '{ "name": "Unagi" }'
      end

      # hard to parse this
      expect(output.tap(&:rewind).read).not_to be_empty
    end
  end
end
